'' RetroTEXT

CON
  
  TV_MODE                       = TV_MODE_NTSC + TV_MODE_FONT_0

OBJ

  tv_SmallFont                  : "Retrotext_002"
  
CON

  TV_MODE_NTSC                  = %0000_0000
  TV_MODE_PAL                   = %0000_0001

  TV_MODE_FAST_UPDATE           = %0000_0000
  TV_MODE_FLICKER_FREE          = %0000_0100

  TV_MODE_COLOR                 = %0000_0000
  TV_MODE_COLOUR                = %0000_0000
  TV_MODE_MONOCHROME            = %0000_1000
  
  TV_MODE_INTERLACED            = %0000_0000
  TV_MODE_NON_INTERLACED        = %0001_0000

  TV_MODE_COMPOSITE             = %0000_0000
  TV_MODE_BASEBAND              = %0000_0000
  TV_MODE_BROADCAST             = %0010_0000

  TV_MODE_FONT_0                = %0000_0000

  TV_MODE_RUNNING               = %1000_0000
    
  RGB_BLACK                     = $02 
  RGB_GREY                      = $03
  RGB_WHITE                     = $06
  RGB_BRIGHT_WHITE              = $07
  RGB_CYAN                      = $3D
  RGB_LIGHT_CYAN                = $3E 
  RGB_LIGHT_GREEN               = $5E 
  RGB_LIGHT_YELLOW              = $8E 
  RGB_LIGHT_RED                 = $CE
  RGB_LIGHT_MAGENTA             = $FE 

  RGB_RED                       = $BC          
  RGB_YELLOW                    = $8D
  RGB_GREEN                     = $5C
  RGB_BLUE                      = $1C
  RGB_MAGENTA                   = $FC
  RGB_LIGHT_BLUE                = $1E

  BLACK                     = $A8 
  WHITE                     = $1F
  BRIGHT_WHITE              = $07
  RED                       = $D2
  CYAN                      = $1C          
  YELLOW                    = $1E
  GREEN                     = $14
  BLUE                      = $A5
  MAGENTA                   = $03
  PEACH                     = $1A
  PINK                      = $29
  ORANGE                    = $12
  AMBER                     = $26 
  SKYBLUE                   = $1D
  PURPLE                    = $01


  CR                            = $0D
  LF                            = $0A
  TAB                           = $09
  BKS                           = $08
  
  'size of the sprite structure
  SPR_SIZE = 8
  
  'maximum number of sprites handled
  SPR_MAX = 8
  
  SPR_X   = 0   'current X position
  SPR_Y   = 1   'current Y position
  SPR_CHR = 2   'current character defined
  SPR_C   = 3   'current character color
  SPR_DIR = 4   'current direction
  SPR_SPD = 5   'current speed
  SPR_CTR = 6   'current counter (for speed)
  SPR_VIS = 7   'is visible
  
  NORTH   = 0
  SOUTH   = 1
  EAST    = 2
  WEST    = 3

VAR

  byte rows
  byte columns

  byte row
  byte column
  byte colour

  byte mode
  
  byte sprite_cnt
  byte spritedat [SPR_SIZE * SPR_MAX]
  word shadowdat [SPR_MAX]

PUB Start(TV_PIN)
  result := Open (TV_PIN, TV_MODE)
  Cls 
  sprite_cnt := 0

PUB out( b )

  case b
    $08   : column--
    $0D   : TxCrLf
    $00   : Cls
    other :      
      TxRawByte( b )

PUB str( txtPtr )

  repeat strsize( txtPtr )
    out( byte[ txtPtr++ ] )

PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10

PUB hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB bin(value, digits)

'' Print a binary number

  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")


' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setBasePin, setMode )

  setMode |= TV_MODE_RUNNING

  if mode <> setMode

    Close
    
    mode := tv_SmallFont.Open( setBasePin, setMode, @PALETTE )
    rows := tv_SmallFont.GetRowCount
    columns := tv_SmallFont.GetColumnCount

    Cls
    
  return mode
  
PUB Close

  if mode
    tv_SmallFont.Close
    mode := 0

' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************
PUB UpdateScreen
   SpriteUpdate
   tv_SmallFont.UpdateScreen
   
PUB TxRawByte( b )

  if column => columns
    TxCrLf
  if row => rows
    Scroll
  tv_SmallFont.PokeChar( row, column, colour, b )
  column++

PUB TxCrLf

  column := 0
  if row++ > rows
    Scroll
    
PUB Cls

  tv_SmallFont.Cls
  colour := 0
  Home

PUB Locate (x,y)

   column := x
   row := y
   
PUB Home

  column := row := 0

PRI Scroll

  repeat while row => rows
    tv_SmallFont.Scroll
    row--
            
PUB Color( useColour )

  colour := useColour

pub redefine(c,c0,c1,c2,c3,c4,c5,c6,c7) | p

   tv_SmallFont.define(c,c0,c1,c2,c3,c4,c5,c6,c7)


pub load (counter,newfont)

   tv_SmallFont.load(counter,newfont)

PUB PokeChar( ccol, rrow, ccolour, cc )  '' Fixed for proper X,Y orientation

   tv_SmallFont.PokeChar(rrow,ccol,ccolour,cc)

PUB GetChar( ccol, rrow )  '' Fixed for proper X,Y orientation

    return tv_SmallFont.GetChar(rrow,ccol)
    
PUB PutChar( ccol, rrow , chr)   '' Fixed for proper X,Y orientation

    tv_SmallFont.PutChar(rrow,ccol,chr)

PUB center(txt) | cal,sz, x

    cal:=strsize(txt)    
    sz:=20-cal/2-1
    repeat x from 0 to sz
      out(" ")      
    str(txt)

PUB newline(Carragereturn) |  x

    repeat x from 1 to CarrageReturn
      out(13)
      
PUB DrawHLine(x, y, l,character) | i
    repeat i from x to l
      pokechar(i, y, colour, character)

PUB DrawVLine(x,y,l,character) | i
    repeat i from y to l
       pokechar(x,i,colour,character)
       

PUB DrawTextAt (x, y,crayon,txtPtr) | i, text_end_pos
        text_end_pos := (strsize( txtPtr ) + (x-1)) 
        repeat i from x to text_end_pos
           'pokechar(i, y, colour, byte[ txtPtr++ ])
           pokechar(i, y, crayon, byte[ txtPtr++ ])
           
PUB DrawRainbowTextAt (x, y,txtPtr) | i, text_end_pos,crayon
        text_end_pos := (strsize( txtPtr ) + (x-1)) 
        repeat i from x to text_end_pos
           crayon:=random(cnt,8,254)
           pokechar(i, y, crayon, byte[ txtPtr++ ])

PUB random(seed, maxValue, minValue)
 
  result := ((||(?seed)) // ((||(maxValue - minValue)) + 1))
 
  if(minValue > maxValue)
 
    result += maxValue
 
  else
  
    result += minValue


    
' *******************************************************************************************************
' * Text Sprite Handling Routines                                                                       *
' *******************************************************************************************************
' to do: figure out how to 'grab' the character that was underneath and replace it when the sprite moves

PUB SpriteAdd (spr_char, spr_color, spr_sx, spr_sy, spr_direction, spr_speed) | ptr, curr_spr, sptr
    if sprite_cnt < SPR_MAX
       curr_spr := sprite_cnt
       ptr := @spritedat + (curr_spr * SPR_SIZE)
       byte[ptr][SPR_X] := spr_sx
       byte[ptr][SPR_Y] := spr_sy
       byte[ptr][SPR_CHR] := spr_char
       byte[ptr][SPR_C] := spr_color
       byte[ptr][SPR_DIR] := spr_direction
       byte[ptr][SPR_SPD] := spr_speed
       byte[ptr][SPR_CTR] := 0
       byte[ptr][SPR_VIS] := 1
       sptr := @shadowdat + (curr_spr * 2)
       word[sptr] := tv_SmallFont.PeekChar(spr_sy, spr_sx)
       tv_SmallFont.PokeChar(spr_sy, spr_sx, spr_color, spr_color)
       sprite_cnt++
       return curr_spr
    else
       return  -1

PUB SpriteHide(sprite_num) | ptr
   ptr := @spritedat + (sprite_num * SPR_SIZE)
   byte[ptr][SPR_VIS] := 0

PRI SpriteUpdate | ptr, i, sptr
   ptr := @spritedat
   repeat i from 0 to sprite_cnt
      if byte[ptr][SPR_VIS] == 1
         if byte[ptr][SPR_SPD] == byte[ptr][SPR_CTR]
            sptr := @shadowdat + (i * 2)
            tv_SmallFont.PutChar(byte[ptr][SPR_Y], byte[ptr][SPR_X], word[sptr])
            case byte[ptr][SPR_DIR]
                  WEST:
                      if byte[ptr][SPR_X] == 0
                         byte[ptr][SPR_X] := 39
                      else
                         byte[ptr][SPR_X]--
                  EAST:
                      if byte[ptr][SPR_X] == 39
                         byte[ptr][SPR_X] := 0
                      else
                         byte[ptr][SPR_X]++
                  SOUTH:
                      if byte[ptr][SPR_Y] == 24
                         byte[ptr][SPR_Y] := 0
                      else
                         byte[ptr][SPR_Y]++
                  NORTH:
                      if byte[ptr][SPR_Y] == 0
                         byte[ptr][SPR_Y] := 24
                      else
                         byte[ptr][SPR_Y]--

            word[sptr] := tv_SmallFont.PeekChar(byte[ptr][SPR_Y], byte[ptr][SPR_X])
            tv_SmallFont.PokeChar(byte[ptr][SPR_Y], byte[ptr][SPR_X], byte[ptr][SPR_C], byte[ptr][SPR_CHR])

            byte[ptr][SPR_CTR] := 0
         else
            byte[ptr][SPR_CTR]++

      ptr += SPR_SIZE
DAT

' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette for Video Drivers                                                                *
' *                                                                                                     *
' *******************************************************************************************************

' Basic QB-style colours
'
'       0 White                 8  Grey
'       1 Blue                  9  Light Blue
'       2 Green                 10 Light Green
'       3 Cyan                  11 Light Cyan
'       4 Red                   12 Light Red
'       5 Magenta               13 Light Magenta
'       6 Yellow                14 Light Yellow
'       7 Bright White          15 White on Red
        
PALETTE byte    RGB_BLACK              ' Background

        byte    RGB_WHITE              ' 0
        byte    RGB_BLUE               ' 1
        byte    RGB_GREEN              ' 2  
        byte    RGB_CYAN               ' 3  
        byte    RGB_RED                ' 4  
        byte    RGB_MAGENTA            ' 5  
        byte    RGB_YELLOW             ' 6  
        byte    RGB_BRIGHT_WHITE       ' 7
          
        byte    RGB_GREY               ' 8 
        byte    RGB_LIGHT_BLUE         ' 9
        byte    RGB_LIGHT_GREEN        ' 10
        byte    RGB_LIGHT_CYAN         ' 11
        byte    RGB_LIGHT_RED          ' 12
        byte    RGB_LIGHT_MAGENTA      ' 13
        byte    RGB_LIGHT_YELLOW       ' 14

        byte    RGB_RED                ' 15 - Background with foreground colour 0
        